<?php
/**
 * @package    com_responsibilities
 * @subpackage models
 * @author     Alexander Grözinger {@link http://www.agmedia.de}
 * @author     Created on Mar-2018
 * @license    GNU/GPL
 * @copyright  AGMedia.de
 */
/*
   _   ___ __  __        _ _           _
  /_\ / __|  \/  |___ __| (_)__ _   __| |___
 / _ \ (_ | |\/| / -_) _` | / _` |_/ _` / -_)
/_/ \_\___|_|  |_\___\__,_|_\__,_(_)__,_\___|
*/

// No direct access to this file
defined('_JEXEC') or die(';-)');

jimport('joomla.application.component.modellist');

class ResponsibilitiesModelResponsibilities extends JModelItem
{

	public function getItem() {
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //Default var's
		$app                = JFactory::getApplication();
		$jinput             = $app->input;
		$menu               = $app->getMenu();
		$active             = $menu->getActive();
		$db                 = JFactory::getDbo();

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//Hole die aktuelle Kategorie-ID
		$id = 0;
        $id = (int) $active->query['cid'];

		if($id == 0) return array();

		////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//Prüfen ob die ausgewählte Kategorie eine LEVEL 1 Kategorie ist
		$categories = new ResponsibilitiesCategories;
		$category	= $categories->get($id);
		
		if($category == NULL OR $category->level != 1) {
			return array(
            	'error' => 'true',
            	'text' => 'Keine Level 1 Kategorie in der Menükonfiguration ausgewählt'
            );	
		}
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Kategorien zur passenden Elternkategorie laden
		$conditions = array(
			$db->quoteName('parent_id') .' = '. $db->quote($category->id),
			$db->quoteName('published') .' = '. $db->quote(1),
			$db->quoteName('level') .' = '. $db->quote(2),
		);
		
		$query = $db->getQuery(true);
		$query  ->select('id, title')
				->from('#__categories');
		
		//Order
		$query  ->order('lft');

		//Where
		$query  ->where($conditions);
		
		$db->setQuery($query);
		$db->execute();
		$count  = $db->getNumRows();
		$categories = $db->loadAssocList();
        
		if ($count < 1) return array();
		/*
		        [0] => Array
                (
                    [id] => 24
                    [title] => Hauptpersonen
                )
		 */
		$categories_to_fill = array();
		foreach($categories as $category) {
		    $categories_to_fill[] = $category['id'];
        }
        

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Die Assignments zu den Kateogieren laden
        $nullDate   = $db->quote($db->getNullDate());
        $nowDate    = $db->quote(JFactory::getDate()->toSql());

		$conditions = array(
			$db->quoteName('cid') .' IN ('. implode(',', $categories_to_fill) .') ',
			$db->quoteName('state') .' = '. $db->quote(1),
		);

		$query = $db->getQuery(true);
		$query  ->select('*')
				->from('#__responsibilities_assignments AS s');

		//Where
		$query  ->where($conditions);
        $query  ->where('(publish_up = ' . $nullDate . ' OR publish_up <= ' . $nowDate . ')');
        $query  ->where('(publish_down = ' . $nullDate . ' OR publish_down >= ' . $nowDate . ')');
        $query	->order('ordering', 'ASC');

		$db->setQuery($query);
		$db->execute();
		$count  = $db->getNumRows();
		$assignments = $db->loadAssocList();
		// echo "<pre>"; print_r($assignments); exit;
		
		/*
		Array
		(
		    [0] => Array
		        (
		            [id] => 1
		            [user_type] => 1
		            [user_id] => 3
		            [cid] => 29
		            [strasse] => 
		            [plz] => 
		            [ort] => 
		            [email] => 
		            [telefon] => 
		            [image] => 
		            [aufgaben] => 
		            [state] => 1
		            [checked_out] => 0
		            [checked_out_time] => 0000-00-00 00:00:00
		            [created] => 2018-03-27 12:59:37
		            [created_by] => 276
		            [modified] => 2018-03-27 16:12:52
		            [modified_by] => 276
		            [publish_up] => 0000-00-00 00:00:00
		            [publish_down] => 0000-00-00 00:00:00
		        )
		
		)		
		*/

		if ($count < 1) return array();

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Die personen zu den Kateogieren hinzufügen
		//TODO: Optimize foreach
        foreach($categories as $cat_key => $category) {
            foreach($assignments as $assignment_key => $assignment) {
                if($assignment['cid'] == $category['id']) {
                	////////////////////////////////////////////////////////////////////////////////////////////
    				// Personendaten holen
					$data = $this->getPersonData($assignment);
                	
                    $categories[$cat_key]['persons'][] = $data;
                    unset($assignments[$assignment_key]);
                }
            }
        }

        return $categories;
	}
	
	protected function getPersonData($assignment) {
		switch($assignment['user_type']) {
			case 1: // Daten kommen von Person der Komponente
				$person_data = $this->getPersonOfComponent($assignment['user_id']);
				// echo "<pre>"; print_r($person_data); exit;
				
				// Prüfe ob Overrides existieren, ansonsten Personendaten verwenden
				foreach($person_data as $field => $value) {
					if(!array_key_exists($field, $assignment)) continue;
					if(!empty($assignment[$field])) continue;
					
					$assignment[$field] = $value;
				}
				
				$assignment['vorname']  = $person_data['vorname'];
				$assignment['nachname'] = $person_data['nachname'];
				
				// echo "<pre>"; print_r($assignment); exit;
				return $assignment;
				break;
			case 2: // Daten kommen von Joomla Konto
				$person_data = $this->getJoomlaUser($assignment['user_id']);
				
				// Prüfe ob Overrides existieren, ansonsten Personendaten verwenden
				foreach($person_data as $field => $value) {
					if(!array_key_exists($field, $assignment)) continue;
					if(!empty($assignment[$field])) continue;
					
					$assignment[$field] = $value;
				}
				
				$name_exploded = explode(' ', $person_data['name']);
				$assignment['vorname']  = $name_exploded[0];
				$assignment['nachname'] = $name_exploded[1];
				return $assignment;
				break;
			case 3: // Daten kommen von Wicked Benutzerkonto
				//TODO
				break;
		}		
	}
	
	protected function getPersonOfComponent($user_id) {
        $db = JFactory::getDbo();
        $conditions = array(
            $db->quoteName('id') 	        .' = '. $db->quote($user_id),
            $db->quoteName('state') 	    .' = '. $db->quote(1),
        );

        $query  = $db->getQuery(true);
        $query  ->select(array('vorname, nachname, strasse, plz, ort, email, aufgaben, telefon, image'))
            	->from($db->quoteName('#__responsibilities'))
            	->where($conditions);
        $db		->setQuery($query);
        $db		->execute();
        $count	= $db->getNumRows();
        $person = $db->loadAssoc();	
        
        if($count < 1) return array();
        return $person;
	}
	
	protected function getJoomlaUser($user_id) {
        $db = JFactory::getDbo();
        $conditions = array(
            $db->quoteName('id') 	        .' = '. $db->quote($user_id),
            $db->quoteName('block') 	    .' = '. $db->quote(0),
        );

        $query  = $db->getQuery(true);
        $query  ->select(array('name', 'email'))
            	->from($db->quoteName('#__users'))
            	->where($conditions);
        $db		->setQuery($query);
        $db		->execute();
        $count	= $db->getNumRows();
        $person = $db->loadAssoc();	
        
        if($count < 1) return array();
        return $person;
	}	
}